#!/bin/sh
#
# Alpine Linux specific package management functions.
#
# Copyright 2024 Andrew Wood
#
# License GPLv3+: GNU GPL version 3 or later; see `docs/COPYING'.
#

# Setup required for all Alpine Linux package actions - set
# "packageManagerFlags" according to whether interactive mode should be
# used; and set "packageListArgs" to the sanitised arguments.
#
uactAlpinePackage () {
	packageManagerFlags=""
	packageListArgs=""

	packageManagerFlags="-q"

	# Set the arguments and environment according to whether interactive
	# mode is enabled.
	test "${optionNonInteractive}" -eq 1 && packageManagerFlags="--no-interactive"

	# Sanitise the input arguments.
	packageListArgs="$(uactSanitisePackageList "$*")"
}

# Install packages $* (actually ${packageListArgs}).
uactAlpinePackageInstall () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} add ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Upgrade packages $* (actually ${packageListArgs}).
uactAlpinePackageUpgrade () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} update >/dev/null
	uactRun "${packageManagerCommand}" ${packageManagerFlags} upgrade ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Downgrade packages $* (actually ${packageListArgs}).
uactAlpinePackageDowngrade () {
	uactError "UNSUPPORTED-ACTION" "${action}"
	return 2
}

# Remove packages $* (actually ${packageListArgs}).
uactAlpinePackageRemove () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} del ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Reinstall packages $* (actually ${packageListArgs}).
uactAlpinePackageReinstall () {
	uactError "UNSUPPORTED-ACTION" "${action}"
	return 2
}

# Output information about packages $* (actually ${packageListArgs}), one
# per line, tab separated columns - package, architecture, version, install
# epoch time, vendor, type, summary.
#
uactAlpinePackageInfo () {
	for packageName in ${packageListArgs}; do
		rawPackageInfo="$(sed -n "/^P:${packageName}\$/,/^\$/p" /lib/apk/db/installed 2>/dev/null)"
		test -n "${rawPackageInfo}" || continue
		packageArchitecture="$(printf "%s\n" "${rawPackageInfo}" | sed -n 's/^A://p' | sed -n 1p | tr '\t' ' ')"
		packageVersion="$(printf "%s\n" "${rawPackageInfo}" | sed -n 's/^V://p' | sed -n 1p | tr '\t' ' ')"
		# NB the installation time is not recorded.
		packageInstallEpoch=0
		packageMaintainer="$(printf "%s\n" "${rawPackageInfo}" | sed -n 's/^m://p' | sed -n 1p | tr '\t' ' ')"
		packageSummary="$(printf "%s\n" "${rawPackageInfo}" | sed -n 's/^T://p' | sed -n 1p | tr '\t' ' ')"
		printf "%s\t%s\t%s\t%d\t%s\t%s\t%s\n" "${packageName}" "${packageArchitecture}" "${packageVersion}" "${packageInstallEpoch}" "${packageMaintainer}" "apk" "${packageSummary}"
	done
	return 0
}

# List the files owned by package $1 (actually ${packageListArgs}).
uactAlpinePackageFiles () {
	queryOutput="$(uactRun "${packageManagerCommand}" ${packageManagerFlags} info --contents ${packageListArgs})"
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	# Package file lists don't seem to contain the leading /, so we need
	# to add it.
	printf "%s\n" "${queryOutput}" | grep -E . | sed '/^[^\/]/s,^,/,'
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List the package which owns file $1.
uactAlpinePackageOwningFile () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} info --who-owns "$1"
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Refresh the cache.
uactAlpinePackageCacheUpdate () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} update
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Clean all caches.
uactAlpinePackageCacheClean () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} cache clean
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List packages ready to upgrade, one per line, tab separated columns -
# package, architecture, target version, repository, type.
uactAlpinePackageUpgradeList () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} update >/dev/null
	upgradePackageList="$(LC_ALL=C uactRun "${packageManagerCommand}" --verbose --no-progress --no-interactive --simulate upgrade)"
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"

	# Turn the output from above into "package targetVersion" pairs.
	upgradePackageList="$(printf "%s\n" "${upgradePackageList}" | sed -n 's/^.* Upgrading //p' | sed -n '/ ([0-9].*-> / {s// /;s/)$//;p}')"

	# Look up the architecture of each package, and produce the
	# formatted output.
	printf "%s\n" "${upgradePackageList}" | {
		defaultArchitecture="$(cat /etc/apk/arch 2>/dev/null)"
		test -n "${defaultArchitecture}" || defaultArchitecture="-"
		while read -r packageName targetVersion; do
			test -n "${packageName}" || continue
			rawPackageInfo="$(sed -n "/^P:${packageName}\$/,/^\$/p" /lib/apk/db/installed 2>/dev/null)"
			packageArchitecture="$(printf "%s\n" "${rawPackageInfo}" | sed -n 's/^A://p' | sed -n 1p | tr '\t' ' ')"
			test -n "${packageArchitecture}" || packageArchitecture="${defaultArchitecture}"
			# Note that no repository details are available.
			printf "%s\t%s\t%s\t%s\t%s\n" "${packageName}" "${packageArchitecture}" "${targetVersion}" "-" "apk"
		done
	}

	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Upgrade all packages.
uactAlpinePackageUpgradeAll () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} update >/dev/null
	uactRun "${packageManagerCommand}" ${packageManagerFlags} upgrade
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List all installed packages.
uactAlpinePackageListInstalled () {
	packageListArgs="$(uactRun "${packageManagerCommand}" ${packageManagerFlags} list --installed)"
	uactAlpinePackageInfo ${packageListArgs}
	return $?
}
