#!/bin/sh
#
# Arch Linux specific package management functions.
#
# Copyright 2024 Andrew Wood
#
# License GPLv3+: GNU GPL version 3 or later; see `docs/COPYING'.
#

# Setup required for all Arch Linux package actions - set
# "packageManagerFlags" according to whether interactive mode should be
# used; and set "packageListArgs" to the sanitised arguments.
#
uactArchlinuxPackage () {
	packageManagerFlags=""
	packageListArgs=""

	packageManagerFlags="--color never"

	# Set the arguments and environment according to whether interactive
	# mode is enabled.
	test "${optionNonInteractive}" -eq 1 && packageManagerFlags="--color never --noconfirm"

	# Sanitise the input arguments.
	packageListArgs="$(uactSanitisePackageList "$*")"
}

# Install packages $* (actually ${packageListArgs}).
uactArchlinuxPackageInstall () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} --sync --refresh --noprogressbar >/dev/null
	uactRun "${packageManagerCommand}" ${packageManagerFlags} --sync --quiet ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Upgrade packages $* (actually ${packageListArgs}).
uactArchlinuxPackageUpgrade () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} --sync --refresh --noprogressbar >/dev/null
	uactRun "${packageManagerCommand}" ${packageManagerFlags} --sync --quiet --needed ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Downgrade packages $* (actually ${packageListArgs}).
uactArchlinuxPackageDowngrade () {
	uactError "UNSUPPORTED-ACTION" "${action}"
	return 2
}

# Remove packages $* (actually ${packageListArgs}).
uactArchlinuxPackageRemove () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} --remove ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Reinstall packages $* (actually ${packageListArgs}).
uactArchlinuxPackageReinstall () {
	# Running "pacman --sync" on a package will also check its
	# checksums, so it will be reinstalled if necessary.
	uactArchlinuxPackageInstall "$@"
	return $?
}

# Output information about packages $* (actually ${packageListArgs}), one
# per line, tab separated columns - package, architecture, version, install
# epoch time, vendor, type, summary.
#
uactArchlinuxPackageInfo () {
	rawPackageInfo="$(LC_ALL=C uactRun "${packageManagerCommand}" ${packageManagerFlags} --query --verbose --info ${packageListArgs})"
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"

	printf "%s\n\n" "${rawPackageInfo}" | awk '
BEGIN {
  pkgname="-"
  arch="-"
  pkgver="-"
  packager="-"
  installdate="never"
}
/^DB Path *: / { dbpath=$NF }
/^Name *: / { pkgname=$NF }
/^Architecture *: / { arch=$NF }
/^Version *: / { pkgver=$NF }
/^Packager *: / { packager=$0; sub(/^[^:]*: /,"",packager) }
/^Description *: / { description=$0; sub(/^[^:]*: /,"",description) }
/^Install Date *: / { installdate=$0; sub(/^[^:]*: /,"",installdate) }
/^$/ && pkgname~/[A-Za-z]/ {
  pkgInfoDir=pkgname "-" pkgver
  gsub(/[^0-9A-Za-z,:_.-]/,"",pkgInfoDir)
  cmd="stat -c %Y " dbpath "/local/" pkgInfoDir " 2>/dev/null"
  cmd | getline installed
  if (installed !~ /[0-9][0-9]/) {
     gsub(/[^0-9A-Za-z,:_.-]/,"",installdate)
     cmd="date -d \"" installdate "\" +%s 2>/dev/null"
     cmd | getline installed
  }
  if (installed !~ /[0-9][0-9]/) {
    installed="-"
  }
  printf "%s\t%s\t%s\t%s\t%s\t%s\t%s\n", pkgname, arch, pkgver, installed, packager, "pacman", description
  pkgname="-"
  arch="-"
  pkgver="-"
  packager="-"
  installdate="never"
}
'

	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List the files owned by package $1 (actually ${packageListArgs}).
uactArchlinuxPackageFiles () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} --query --quiet --list ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List the package which owns file $1.
uactArchlinuxPackageOwningFile () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} --query --quiet --owns "$1"
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Refresh the cache.
uactArchlinuxPackageCacheUpdate () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} --sync --refresh
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Clean all caches.
uactArchlinuxPackageCacheClean () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} --sync --clean --quiet
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List packages ready to upgrade, one per line, tab separated columns -
# package, architecture, target version, repository, type.
uactArchlinuxPackageUpgradeList () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} --sync --refresh --noprogressbar >/dev/null
	upgradePackageList="$(LC_ALL=C uactRun "${packageManagerCommand}" ${packageManagerFlags} --query --quiet --upgrades)"
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	if test -n "${upgradePackageList}"; then
		uactRun "${packageManagerCommand}" ${packageManagerFlags} --sync --print --print-format "$(printf '%%n\t%%a\t%%v\t%%r\tpacman\n')" ${upgradePackageList}
		cmdExitStatus=$?
		uactDebug "command exit status: ${cmdExitStatus}"
	fi
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Upgrade all packages.
uactArchlinuxPackageUpgradeAll () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} --sync --refresh --sysupgrade
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List all installed packages.
uactArchlinuxPackageListInstalled () {
	uactArchlinuxPackageInfo
	return $?
}
