#!/bin/sh
#
# Main program entry point.
#
# Copyright 2023-2024 Andrew Wood
#
# License GPLv3+: GNU GPL version 3 or later; see `docs/COPYING'.
#

# Adjust the default path.
export PATH="/sbin:/bin:/usr/sbin:/usr/bin:$PATH"

# Load the output, function override, and extension functions.
. "${COMPONENTDIR}/output.sh"
. "${COMPONENTDIR}/override.sh"
. "${COMPONENTDIR}/extensions.sh"

action=""
actionClass=""
optionNonInteractive=0
optionVerbose=0
optionQuiet=0
optionJson=0
stdoutFile="-"
stderrFile="-"
teeFile=""
minArgs=0
maxArgs=0
outputDataType="string"
isExtension=0

exitStatus=0
actionOutput=""


# Default to non-interactive mode if stdin is not a terminal.
test -t 0 || optionNonInteractive=1

# Parse the command-line options to apply options and select the action.
while test -n "$1" && test -z "${action}" && test "${exitStatus}" -eq 0; do
	arg="$1"
	shift
	case "${arg}" in
	"-h"|"--help"|"help")		action="help" ;;
	"-V"|"--version"|"version")	action="version" ;;
	"-n"|"--non-interactive")	optionNonInteractive=1 ;;
	"-i"|"--interactive")		optionNonInteractive=0 ;;
	"-v"|"--verbose")		optionVerbose=$((1+optionVerbose)) ;;
	"-vv"|"--debug")		optionVerbose=$((2+optionVerbose)) ;;
	"-q"|"--quiet")			optionQuiet=$((1+optionQuiet)) ;;
	"-qq"|"--very-quiet")		optionQuiet=$((2+optionQuiet)) ;;
	"-j"|"--json")			optionJson=1 ;;
	"-o"|"--output"|"--stdout")	test -n "$1" && { stdoutFile="$1"; shift; } || { uactError "MISSING-OPTION-ARGUMENT" "${arg}"; exitStatus=1; } ;;
	"-t"|"--tee")			test -n "$1" && { teeFile="$1"; shift; }    || { uactError "MISSING-OPTION-ARGUMENT" "${arg}"; exitStatus=1; } ;;
	"-e"|"--error"|"--stderr")	test -n "$1" && { stderrFile="$1"; shift; } || { uactError "MISSING-OPTION-ARGUMENT" "${arg}"; exitStatus=1; } ;;
	"--"*)				uactError "UNKNOWN-OPTION" "${arg}"; exitStatus=1 ;;
	"-"[a-z]*)
		for optionLetter in $(echo "${arg#-}" | sed 's,\(.\),\1 ,g'); do
			case "${optionLetter}" in
			'h')	action="help" ;;
			'V')	action="version" ;;
			'n')	optionNonInteractive=1 ;;
			'i')	optionNonInteractive=0 ;;
			'v')	optionVerbose=$((1+optionVerbose)) ;;
			'q')	optionQuiet=$((1+optionQuiet)) ;;
			'j')	optionJson=1 ;;
			'o')	test -n "$1" && { stdoutFile="$1"; shift; } || { uactError "MISSING-OPTION-ARGUMENT" "-${optionLetter}"; exitStatus=1; } ;;
			't')	test -n "$1" && { teeFile="$1"; shift; }    || { uactError "MISSING-OPTION-ARGUMENT" "-${optionLetter}"; exitStatus=1; } ;;
			'e')	test -n "$1" && { stderrFile="$1"; shift; } || { uactError "MISSING-OPTION-ARGUMENT" "-${optionLetter}"; exitStatus=1; } ;;
			*)	uactError "UNKNOWN-OPTION" "${optionLetter}"; exitStatus=1 ;;
			esac
			test "${exitStatus}" -eq 0 || break
		done
		;;
	"-"*)				uactError "UNKNOWN-OPTION" "${arg}"; exitStatus=1 ;;
	"package-install")		minArgs=1; maxArgs=0; action="package-install" ;;
	"package-update")		minArgs=1; maxArgs=0; action="package-upgrade" ;;
	"package-upgrade")		minArgs=1; maxArgs=0; action="package-upgrade" ;;
	"package-downgrade")		minArgs=1; maxArgs=0; action="package-downgrade" ;;
	"package-remove")		minArgs=1; maxArgs=0; action="package-remove" ;;
	"package-reinstall")		minArgs=1; maxArgs=0; action="package-reinstall" ;;
	"package-info")			minArgs=1; maxArgs=0; action="package-info";           outputDataType="list package,architecture,version,installEpoch:I,vendor,type,summary" ;;
	"package-files")		minArgs=1; maxArgs=0; action="package-files";          outputDataType="strings" ;;
	"package-owning-file")		minArgs=1; maxArgs=0; action="package-owning-file" ;;
	"package-cache-update")		minArgs=0; maxArgs=0; action="package-cache-update" ;;
	"package-cache-clean")		minArgs=0; maxArgs=0; action="package-cache-clean" ;;
	"package-upgrade-list")		minArgs=0; maxArgs=0; action="package-upgrade-list";   outputDataType="list package,architecture,targetVersion,repository,type" ;;
	"package-upgrade-all")		minArgs=0; maxArgs=0; action="package-upgrade-all" ;;
	"package-list-installed")	minArgs=0; maxArgs=0; action="package-list-installed"; outputDataType="list package,architecture,version,installEpoch:I,vendor,type,summary" ;;
	"service-start")		minArgs=1; maxArgs=1; action="service-start" ;;
	"service-stop")			minArgs=1; maxArgs=1; action="service-stop" ;;
	"service-restart")		minArgs=1; maxArgs=1; action="service-restart" ;;
	"service-try-restart")		minArgs=1; maxArgs=1; action="service-try-restart" ;;
	"service-reload")		minArgs=1; maxArgs=1; action="service-reload" ;;
	"service-enable")		minArgs=1; maxArgs=1; action="service-enable" ;;
	"service-disable")		minArgs=1; maxArgs=1; action="service-disable" ;;
	"service-mask")			minArgs=1; maxArgs=1; action="service-mask" ;;
	"service-unmask")		minArgs=1; maxArgs=1; action="service-unmask" ;;
	"service-is-known")		minArgs=1; maxArgs=1; action="service-is-known";   outputDataType="boolean" ;;
	"service-is-running")		minArgs=1; maxArgs=1; action="service-is-running"; outputDataType="boolean" ;;
	"service-is-enabled")		minArgs=1; maxArgs=1; action="service-is-enabled"; outputDataType="boolean" ;;
	"service-is-masked")		minArgs=1; maxArgs=1; action="service-is-masked";  outputDataType="boolean" ;;
	"service-list-all")		minArgs=0; maxArgs=0; action="service-list-all";   outputDataType="list service,boot,active,description" ;;
	"system-info")			minArgs=0; maxArgs=0; action="system-info";        outputDataType="list machineIdHash,dnsHost,dnsDomain,kernelName,kernelVersion,arch,ramSize:I,cpuCount:I,ipAddresses,osName,osId,osVersionId,osBuildId,osLike,osCpeName,osPrettyName" ;;
	"listening-ports")		minArgs=0; maxArgs=0; action="listening-ports";    outputDataType="list protocol,ip,port:I,pid:I,command" ;;
	*)
		uactLoadExtensions
		matchingExtension="$(printf "%s\n" "${extensionList}" | awk -F "\t" -v action="${arg}" '$1==action { print }' | sed -n 1p)"
		if ! test -n "${matchingExtension}"; then
			uactError "UNKNOWN-ACTION" "${arg}"
			exitStatus=1
		fi
		minArgs="$(printf "%s\n" "${matchingExtension}" | awk -F "\t" '{print $2}')"
		test -n "${minArgs}" || minArgs=0
		maxArgs="$(printf "%s\n" "${matchingExtension}" | awk -F "\t" '{print $3}')"
		test -n "${maxArgs}" || maxArgs=0
		outputDataType="$(printf "%s\n" "${matchingExtension}" | awk -F "\t" '{print $5}')"
		test -n "${outputDataType}" || outputDataType="string"
		action="${arg}"
		isExtension=1
		;;
	esac
done

# Load the JSON functions if JSON output is to be written.
test "${optionJson}" -eq 1 && . "${COMPONENTDIR}/json.sh"

# Set up an exit handler for JSON output during argument checks.
test "${optionJson}" -eq 1 && trap 'uactJsonOutput "${actionOutput}"' EXIT

# Exit now if there was a problem parsing the arguments.
test "${exitStatus}" -eq 0 || exit ${exitStatus}

# Redirect standard output if applicable.
test -n "${stdoutFile}" && test "${stdoutFile}" != "-" && exec >>"${stdoutFile}"

# Redirect standard error if applicable.
# Note we use fd 3 as well - see uactRun in output.sh.
test -n "${stderrFile}" && test "${stderrFile}" != "-" && { exec 2>>"${stderrFile}"; exec 3>>"${stderrFile}"; }

# Early exit if no action was chosen.
test -z "${action}" && { uactError "NO-ACTION"; exitStatus=1; exit ${exitStatus}; }

# Early exit if the number of parameters is incorrect.
test "${minArgs}" -gt 0 && test -z "$1" && { uactError "MISSING-ARGUMENT" "${action}"; exitStatus=1; exit ${exitStatus}; }
test "${minArgs}" -eq 0 && test "${maxArgs}" -eq 0 && test -n "$1" && { uactError "UNEXPECTED-ARGUMENT" "${action}"; exitStatus=1; exit ${exitStatus}; }

# Store the arguments as a string for hooks to view.
actionArguments="$*"

# Set the action class to the first word of the action.
actionClass="${action%%-*}"

# Camel-case the action and class, so "service-is-masked" becomes
# "uactServiceIsMasked", to get the appropriate function name to call.
functionPrefix="uact"
test "${isExtension}" -eq 1 && functionPrefix="uactExtension"
actionFunction="${functionPrefix}"
for word in $(echo "${action}" | tr '-' ' '); do
	actionFunction="${actionFunction}$(echo "${word}" | cut -b 1 | tr '[:lower:]' '[:upper:]')${word#?}"
done
# The action class is just one word.
actionClassFunction="${functionPrefix}$(echo "${actionClass}" | cut -b 1 | tr '[:lower:]' '[:upper:]')${actionClass#?}"

# Honour the "--quiet" and "--quiet --quiet" settings.
test "${optionQuiet}" -gt 0 && { exec 2>/dev/null; exec 3>/dev/null; }
test "${optionQuiet}" -gt 1 && exec >/dev/null

uactDebug "actionClass=[${actionClass}], actionClassFunction=[${actionClassFunction}]"
uactDebug "action=[${action}], actionFunction=[${actionFunction}]"
uactDebug "actionArguments=[$*]"
uactDebug "optionNonInteractive=${optionNonInteractive}"
uactDebug "optionVerbose=${optionVerbose}"
uactDebug "optionQuiet=${optionQuiet}"
uactDebug "optionJson=${optionJson}"
uactDebug "stdoutFile=${stdoutFile}"
uactDebug "stderrFile=${stderrFile}"
uactDebug "teeFile=${teeFile}"
uactDebug "outputDataType=${outputDataType}"

# Remove the JSON output exit handler, in case any hooks or any action
# functions are going to set exit traps themselves.
test "${optionJson}" -eq 1 && trap '' EXIT

# Function to run a command and use signals to inform the parent process if
# it exited non-zero, so that we can catch when the first command in a
# pipeline exits non-zero, since a pipeline's exit status is that of the
# last command in it.
#
# For example, with "do-something | tee FILE", the "tee" likely exits 0, so
# we never know whether "do-something" failed.
#
# Calling "uactCaptureExitViaUSR do-something | tee FILE" will use signals to set
# $uactCapturedExitUSR to 1 if "do-something" exits 1, or 2 if it exits anything
# higher than 1.
#
# The caller must set $uactCapturedExitUSR to 0 before running the pipeline.
#
uactCaptureExitViaUSR () {
	"$@"
	rc="$?"
	test "${rc}" -eq 1 && kill -USR1 $$
	test "${rc}" -gt 1 && kill -USR2 $$
	return "${rc}"
}
trap 'uactCapturedExitUSR=1' USR1
trap 'uactCapturedExitUSR=2' USR2

# Function to capture the output of a command into $actionOutput, returning
# the command's exit status (where any status above 2 is represented as 2).
#
# If $teeFile is set, then the output is copied to that destination with
# "tee" while being captured.
#
# Because the output is being captured, any interactive prompts won't be
# seen by the operator.  For this reason, if stdin is a terminal and "-n"
# was not given, and "-qq" was also not given, and this is an action with a
# "string" output data type, then another "tee" is used to copy output to
# /dev/tty.
#
uactCaptureOutput () {
	needTerminalCopy=0
	test -t 0 \
	&& test "${optionNonInteractive}" -eq 0 \
	&& test "${optionQuiet}" -lt 2 \
	&& test "${outputDataType}" = "string" \
	&& needTerminalCopy=1

	test "${action}" = "help" && needTerminalCopy=0
	test "${action}" = "version" && needTerminalCopy=0

	uactDebug "needTerminalCopy=${needTerminalCopy}"

	# Clear the exit status prior to possible adjustment by signals
	# received by the uactCaptureExitViaUSR call.
	uactCapturedExitUSR=0

	if test -n "${teeFile}"; then
		# Copy to $teeFile via "tee".
		if test "${needTerminalCopy}" -eq 1; then
			# Also copy to /dev/tty.
			actionOutput="$(uactCaptureExitViaUSR "$@" | tee -a "${teeFile}" /dev/tty)"
		else
			actionOutput="$(uactCaptureExitViaUSR "$@" | tee -a "${teeFile}")"
		fi
	elif test "${needTerminalCopy}" -eq 1; then
		# Copy to /dev/tty via "tee".
		actionOutput="$(uactCaptureExitViaUSR "$@" | tee -a /dev/tty)"
	else
		# No copy needed, just capture the output.
		actionOutput="$(uactCaptureExitViaUSR "$@")"
	fi

	return ${uactCapturedExitUSR}
}

# Run the pre-action hooks.
. "${COMPONENTDIR}/hooks.sh"
uactHooks "pre" "all" "$@"
uactHooks "pre" "${actionClass}" "$@"
test "${actionClass}" = "${action}" || uactHooks "pre" "${action}" "$@"

# Run the class action, loading the relevant file first.
if test -n "${actionClass}" && test "${isExtension}" -eq 0 && test -e "${COMPONENTDIR}/${actionClass}.sh"; then
	uactDebug "loading action class script: ${COMPONENTDIR}/${actionClass}.sh"
	. "${COMPONENTDIR}/${actionClass}.sh"
fi
if type "${actionClassFunction}" >/dev/null 2>&1; then
	uactDebug "running action class function: ${actionClassFunction}"

	if test "${action}" = "${actionClass}"; then
		# If the action class is the same as the action, capture the
		# output so we can transform it later if necessary.  Note
		# that this means any environmental side effects won't
		# persist in the main thread.
		uactCaptureOutput "${actionClassFunction}" "$@"
		returnCode=$?
	else
		"${actionClassFunction}" "$@"
		returnCode=$?
	fi

	test "${returnCode}" -gt 0 && test "${exitStatus}" -eq 0 && exitStatus="${returnCode}"
	uactDebug "exitStatus=${exitStatus}"
fi

# Run the specific action.
if ! test "${action}" = "${actionClass}"; then
	if test -e "${COMPONENTDIR}/${action}.sh"; then
		uactDebug "loading action script: ${COMPONENTDIR}/${action}.sh"
		. "${COMPONENTDIR}/${action}.sh"
	fi
	if type "${actionFunction}" >/dev/null 2>&1; then
		uactDebug "running action function: ${actionFunction}"
		# Capture the output of the action function so we can
		# transform it later if necessary.  This does mean that any
		# environmental side effects won't persist in the main
		# thread.
		previousActionOutput="${actionOutput}"

		uactCaptureOutput "${actionFunction}" "$@"
		returnCode=$?

		test -n "${previousActionOutput}" && actionOutput="$(printf "%s\n%s\n" "${previousActionOutput}" "${actionOutput}")"

		test "${returnCode}" -gt 0 && test "${exitStatus}" -eq 0 && exitStatus="${returnCode}"
		uactDebug "exitStatus=${exitStatus}"
	else
		uactError "UNSUPPORTED-ACTION" "${action}"
		exitStatus=2
	fi
fi

# Run the post-action hooks.
test "${actionClass}" = "${action}" || uactHooks "post" "${action}" "$@"
uactHooks "post" "${actionClass}" "$@"
uactHooks "post" "all" "$@"

# If the output data format is just a big string (not formatted data or a
# list), run it through "col -b" so that progress bars and so on don't take
# up a lot of space in the output.
#
if test "${outputDataType}" = "string" && test -n "${actionOutput}" && command -v col >/dev/null 2>&1; then
	reformattedOutput="$(printf "%s\n" "${actionOutput}" | col -b 2>/dev/null)"
	if test -n "${reformattedOutput}"; then
		uactDebug "reformatting output with \`col -b'"
		actionOutput="${reformattedOutput}"
	else
		uactDebug "reformatting with \`col -b' resulted in no output - not reformatting"
	fi
fi

# Write the action output, in JSON form or in text.
if test "${optionJson}" -eq 1; then
	uactJsonOutput "${actionOutput}"
elif test -n "${actionOutput}"; then
	case "${outputDataType}" in
	"list"*)
		# Prefix tab-separated data with a header line containing
		# the field names.
		headerLine="$(printf "%s\n" "${outputDataType#list }" | tr ',' '\n' | sed 's/:.*$//' | tr '\n' '\t' | sed 's/.$//')"
		printf "%s\n" "${headerLine}"
		;;
	esac
	printf "%s\n" "${actionOutput}"
fi

exit ${exitStatus}
