#!/bin/sh
#
# Debian-specific package management functions.
#
# Copyright 2023-2024 Andrew Wood
#
# License GPLv3+: GNU GPL version 3 or later; see `docs/COPYING'.
#

# Setup required for all Debian package actions - set "packageManagerFlags"
# and DEBIAN_FRONTEND according to whether interactive mode should be used;
# and set "packageListArgs" to the sanitised arguments.
#
uactDebianPackage () {
	packageManagerFlags=""
	packageListArgs=""

	# Set the arguments and environment according to whether interactive
	# mode is enabled.
	DEBIAN_FRONTEND="readline"
	test "${optionNonInteractive}" -eq 1 && packageManagerFlags="-y"
	test "${optionNonInteractive}" -eq 1 && DEBIAN_FRONTEND="noninteractive"
	export DEBIAN_FRONTEND

	# Sanitise the input arguments.
	packageListArgs="$(uactSanitisePackageList "$*")"
}

# Install packages $* (actually ${packageListArgs}).
uactDebianPackageInstall () {
	uactRun "${packageManagerCommand}" --quiet update >/dev/null
	uactRun "${packageManagerCommand}" ${packageManagerFlags} --no-upgrade install ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Upgrade packages $* (actually ${packageListArgs}).
uactDebianPackageUpgrade () {
	uactRun "${packageManagerCommand}" --quiet update >/dev/null
	uactRun "${packageManagerCommand}" ${packageManagerFlags} --only-upgrade install ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Downgrade packages $* (actually ${packageListArgs}).
uactDebianPackageDowngrade () {
	uactRun "${packageManagerCommand}" --quiet update >/dev/null
	uactRun "${packageManagerCommand}" ${packageManagerFlags} --allow-downgrades install ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Remove packages $* (actually ${packageListArgs}).
uactDebianPackageRemove () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} remove ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Reinstall packages $* (actually ${packageListArgs}).
uactDebianPackageReinstall () {
	uactRun "${packageManagerCommand}" --quiet update >/dev/null
	uactRun "${packageManagerCommand}" ${packageManagerFlags} --reinstall install ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Output information about packages $* (actually ${packageListArgs}), one
# per line, tab separated columns - package, architecture, version, install
# epoch time, vendor, type, summary.
#
uactDebianPackageInfo () {
	# Include the status in the query so we can check the package is
	# installed (second character is "i") and skip those that are not. 
	# Also, use \r as a field separator since it's unlikely to appear in
	# the name or synopsis, so we can strip tabs from inside columns.
	uactRun "${packageInfoTool}" --show --showformat='${db:Status-Abbrev}\r${Package}\r${Architecture}\r${Version}\r${db-fsys:Last-Modified}\r${Maintainer}\rdeb\r${binary:Synopsis}\n' ${packageListArgs} 2>/dev/null \
	| sed -n 's/^.i.\r//p' \
	| tr '\t' ' ' \
	| tr '\r' '\t'
	return 0
}

# List the files owned by package $1 (actually ${packageListArgs}).
uactDebianPackageFiles () {
	uactRun "${packageInfoTool}" --listfiles ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List the package which owns file $1.
uactDebianPackageOwningFile () {
	queryOutput=$(LC_ALL=C uactRun "${packageInfoTool}" --search "$1")
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 || return 1
	printf "%s\n" "${queryOutput}" | grep -Ev '^diversion by' | cut -d : -f 1 | tr ',' '\n' | sed 's,^ *,,' | sort -u
	return 0
}

# Refresh the cache.
uactDebianPackageCacheUpdate () {
	uactRun "${packageManagerCommand}" update
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Clean all caches.
uactDebianPackageCacheClean () {
	uactRun "${packageManagerCommand}" clean
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List packages ready to upgrade.
uactDebianPackageUpgradeList () {
	uactRun "${packageManagerCommand}" --quiet update >/dev/null
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 || return 1

# The output of "apt list" is easier to parse.
#	queryOutput=$(uactRun "${packageManagerCommand}" upgrade --simulate)

	queryOutput=$(LC_ALL=C uactRun apt list --upgradeable 2>/dev/null)
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 || return 1

# This partially parses "apt-get upgrade --simulate" but is incomplete.
#	printf "%s\n" "${queryOutput}" | sed -n '/^Inst /{s///;s/ .*$//;p}' | sort -u

	# Parse "apt list --upgradeable" output to get package, arch,
	# target-version, repo, type.
	printf "%s\n" "${queryOutput}" \
	| awk '/^[A-Za-z0-9_.-]+\//{split($1,a,/\//);if (a[2]!="") printf "%s\t%s\t%s\t%s\t%s\n",a[1],$3,$2,a[2],"deb"}' \
	| sort -u
	return 0
}

# Upgrade all packages.
uactDebianPackageUpgradeAll () {
	uactRun "${packageManagerCommand}" --quiet update >/dev/null
	uactRun "${packageManagerCommand}" ${packageManagerFlags} upgrade
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List all installed packages.
uactDebianPackageListInstalled () {
	uactDebianPackageInfo
	return $?
}
