#!/bin/sh
#
# FreeBSD-specific package management functions.
#
# Copyright 2024 Andrew Wood
#
# License GPLv3+: GNU GPL version 3 or later; see `docs/COPYING'.
#

# Setup required for all FreeBSD package actions - set "packageManagerFlags"
# and ASSUME_ALWAYS_YES according to whether interactive mode should be
# used; and set "packageListArgs" to the sanitised arguments.
#
uactFreebsdPackage () {
	packageManagerFlags=""
	packageListArgs=""

	# Set the arguments and environment according to whether interactive
	# mode is enabled.
	test "${optionNonInteractive}" -eq 1 && packageManagerFlags="--yes --quiet"
	test "${optionNonInteractive}" -eq 1 && { ASSUME_ALWAYS_YES="YES"; export ASSUME_ALWAYS_YES; }

	# Sanitise the input arguments.
	packageListArgs="$(uactSanitisePackageList "$*")"
}

# Install packages $* (actually ${packageListArgs}).
uactFreebsdPackageInstall () {
	uactRun "${packageManagerCommand}" install ${packageManagerFlags} ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Upgrade packages $* (actually ${packageListArgs}).
uactFreebsdPackageUpgrade () {
	uactRun "${packageManagerCommand}" upgrade ${packageManagerFlags} ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Downgrade packages $* (actually ${packageListArgs}).
uactFreebsdPackageDowngrade () {
	uactError "UNSUPPORTED-ACTION" "${action}"
	return 2
}

# Remove packages $* (actually ${packageListArgs}).
uactFreebsdPackageRemove () {
	uactRun "${packageManagerCommand}" delete ${packageManagerFlags} ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Reinstall packages $* (actually ${packageListArgs}).
uactFreebsdPackageReinstall () {
	uactRun "${packageManagerCommand}" install ${packageManagerFlags} --force ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Output information about packages $* (actually ${packageListArgs}), one
# per line, tab separated columns - package, architecture, version, install
# epoch time, vendor, type, summary.
#
uactFreebsdPackageInfo () {
	packageQueryRegex="."
	test -n "${packageListArgs}" && packageQueryRegex="^($(printf '%s\n' "${packageListArgs}" | tr '\n' '|' | sed 's/|$//'))\$"
	uactRun pkg query -x '%n\r%q\r%v\r%t\r%m\rpkg\r%c' "${packageQueryRegex}" \
	| awk -F "\r" -v OFS="\r" '{sub(/^.*:/,"",$2);print}' \
	| tr '\t' ' ' \
	| tr '\r' '\t'
	return 0
}

# List the files owned by package $1 (actually ${packageListArgs}).
uactFreebsdPackageFiles () {
	uactRun "${packageManagerCommand}" info --quiet --list ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List the package which owns file $1.
uactFreebsdPackageOwningFile () {
	queryOutput=$(uactRun "${packageManagerCommand}" which --quiet "$1")
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 || return 1
	printf "%s\n" "${queryOutput}" | sed 's,-[^-]*$,,' | sort -u
	return 0
}

# Refresh the cache.
uactFreebsdPackageCacheUpdate () {
	uactRun "${packageManagerCommand}" update --quiet
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Clean all caches.
uactFreebsdPackageCacheClean () {
	uactRun "${packageManagerCommand}" clean ${packageManagerFlags}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List packages ready to upgrade, one per line, tab separated columns -
# package, architecture, target version, repository, type.
uactFreebsdPackageUpgradeList () {
	upgradeOutput="$(LC_ALL=C uactRun "${packageManagerCommand}" upgrade ${packageManagerFlags} --dry-run 2>/dev/null)"
	filteredOutput="$(printf "%s\n" "${upgradeOutput}" | grep -E '^[[:space:]]*[^: ]+: [0-9][^ ]* -> [0-9]' | sed 's/://')"
	upgradePackageList="$(printf "%s\n" "${filteredOutput}" | awk '{print $1}')"
	test -n "${upgradePackageList}" || return 0

	packageQueryRegex="^($(printf '%s\n' "${upgradePackageList}" | tr '\n' '|' | sed 's/|$//'))\$"
	packageQueryResult="$(uactRun pkg query -x '%n %q %R' "${packageQueryRegex}" 2>/dev/null | awk '{sub(/^.*:/,"",$2);print}')"

	printf "%s\n" "${filteredOutput}" \
	| {
		while read -r packageName currentVersion arrow targetVersion; do
			packageArchitecture="$(printf "%s\n" "${packageQueryResult}" | awk -v "name=${packageName}" '$1==name {print $2}')"
			packageRepository="$(printf "%s\n" "${packageQueryResult}" | awk -v "name=${packageName}" '$1==name {print $3}')"
			test -n "${packageArchitecture}" || packageArchitecture="?"
			test -n "${packageRepository}" || packageRepository="?"
			printf "%s\t%s\t%s\t%s\t%s\n" "${packageName}" "${packageArchitecture}" "${packageRepository}" "${targetVersion}" "pkg"
		done
	}

	return 0
}

# Upgrade all packages.
uactFreebsdPackageUpgradeAll () {
	uactRun "${packageManagerCommand}" upgrade ${packageManagerFlags}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List all installed packages.
uactFreebsdPackageListInstalled () {
	uactFreebsdPackageInfo
	return $?
}
