#!/bin/sh
#
# Service management functions specific to FreeBSD.
#
# Copyright 2024 Andrew Wood
#
# License GPLv3+: GNU GPL version 3 or later; see `docs/COPYING'.
#

# Start service $1.
uactFreebsdServiceStart () {
	if uactRun service "$1" enabled >/dev/null 2>&1; then
		uactRun service "$1" start >/dev/null
		cmdExitStatus=$?
	else
		uactRun service "$1" onestart >/dev/null
		cmdExitStatus=$?
	fi
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Stop service $1.
uactFreebsdServiceStop () {
	uactRun service "$1" stop >/dev/null
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Stop, then start, service $1.
uactFreebsdServiceRestart () {
	if uactRun service "$1" enabled >/dev/null 2>&1; then
		uactRun service "$1" restart >/dev/null
		cmdExitStatus=$?
	else
		uactRun service "$1" onerestart >/dev/null
		cmdExitStatus=$?
	fi
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Restart service $1 if it is currently running.
uactFreebsdServiceTryRestart () {
	if uactRun service "$1" status >/dev/null 2>&1; then
		if uactRun service "$1" enabled >/dev/null 2>&1; then
			uactRun service "$1" restart >/dev/null
			cmdExitStatus=$?
		else
			uactRun service "$1" onerestart >/dev/null
			cmdExitStatus=$?
		fi
		uactDebug "command exit status: ${cmdExitStatus}"
	else
		uactDebug "service not running, so no restart needed"
		cmdExitStatus=0
	fi
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Tell service $1 to reload its configuration.
uactFreebsdServiceReload () {
	if uactRun service "$1" oneextracommands | tr ' ' '\n' | grep -Fqx 'reload'; then
		uactRun service "$1" reload >/dev/null
		cmdExitStatus=$?
	else
		uactError "UNSUPPORTED-ACTION-FOR-SERVICE" "${action}" "$1"
		return 2
	fi
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Enable service $1 so it will start on boot.
uactFreebsdServiceEnable () {
	uactRun service "$1" enable >/dev/null
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Disable service $1 so it will not start on boot.
uactFreebsdServiceDisable () {
	uactRun service "$1" disable >/dev/null
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Mask service $1 so it will not start on boot and cannot be started.
uactFreebsdServiceMask () {
	# Service masking is not supported on FreeBSD.
	uactError "UNSUPPORTED-ACTION" "${action}"
	return 2
}

# Unmask service $1 so it can be started and may start on boot.
uactFreebsdServiceUnmask () {
	# Service masking is not supported on FreeBSD.
	uactError "UNSUPPORTED-ACTION" "${action}"
	return 2
}

# Report "true" or "false" depending on whether service $1 exists.
uactFreebsdServiceIsKnown () {
	uactRun service -r | awk -F '/' '{print $NF}' | grep -Fqx "$1" && { echo "true"; return 0; }
	echo "false"
	return 1
}

# Report "true" or "false" depending on whether service $1 is running.
uactFreebsdServiceIsRunning () {
	uactRun service "$1" status >/dev/null 2>&1 && { echo "true"; return 0; }
	echo "false"
	return 1
}

# Report "true" or "false" depending on whether service $1 is enabled.
uactFreebsdServiceIsEnabled () {
	uactRun service "$1" enabled >/dev/null && { echo "true"; return 0; }
	echo "false"
	return 1
}

# Report "true" or "false" depending on whether service $1 is masked.
uactFreebsdServiceIsMasked () {
	# Service masking is not supported on FreeBSD.
	uactError "UNSUPPORTED-ACTION" "${action}"
	return 2
}

# Output one line per service containing the fields "service" (name), "boot"
# (enabled, disabled, masked), "active" (inactive, activating, active),
# "description", TAB separated.
uactFreebsdServiceListAll () {
	uactRun service -r | awk -F '/' '{print $NF}' | sort -u | \
	{
	while read -r serviceName; do
		test -n "${serviceName}" || continue

		bootStatus="disabled"
		uactRun service "${serviceName}" enabled >/dev/null 2>&1 && bootStatus="enabled"

		activeStatus="inactive"
		uactRun service "${serviceName}" status >/dev/null 2>&1 && activeStatus="active"

		serviceDescription="$(uactRun service "${serviceName}" describe | tr '\t\n' '  ')"
		test -n "${serviceDescription}" || serviceDescription="-"

		printf "%s\t%s\t%s\t%s\n" "${serviceName}" "${bootStatus}" "${activeStatus}" "${serviceDescription}"
	done
	}

	return 0
}
