#!/bin/sh
#
# Functions for JSON formatted output.
#
# Copyright 2024 Andrew Wood
#
# License GPLv3+: GNU GPL version 3 or later; see `docs/COPYING'.
#

# Output $1 with JSON escaping, such that it could be surrounded by double
# quotes to form a valid JSON string.
#
# Note that this will strip the trailing newline, if there is one.
#
uactJsonEscapeString () {
	rawString="$1"
	test -n "${rawString}" || return
	# OpenBSD sed doesn't seem to get on with 's,\\,\\\\,g'.
	# printf "%s\n" "${rawString}" | sed -e 's,\\,\\\\,g;s,",\\",g;s,\r,\\r,g;s,\t,\\t,g;s,$,\\n,' | tr -d '\n' | sed 's,\\n$,,'
	printf "%s\n" "${rawString}" \
	| awk '{ gsub(/\\/,"\\\\",$0); gsub(/"/,"\\\"",$0); gsub(/\t/,"\\t",$0); gsub(/\r/,"\\r",$0); print $0 "\\n" }' \
	| tr -d '\n' \
	| sed 's,\\n$,,'
}

# Split $1 into separate lines and output a JSON array containing each line
# as a quoted string.
#
uactJsonStringList () {
	rawStringList="$1"
	if ! test -n "${rawStringList}"; then
		printf "%s" "[]"
		return
	fi
	printf "%s" "["
	printf "%s\n" "${rawStringList}" | {
		firstLine=1
		while read -r singleLine; do
			if test "${firstLine}" -eq 1; then
				printf "\n"
				firstLine=0
			else
				printf ",\n"
			fi
			printf "    \"%s\"" "$(uactJsonEscapeString "${singleLine}")"
		done
		test "${firstLine}" -eq 1 || printf "\n  "
	}
	printf "%s" "]"
}

# Split $2 into separate lines and output a JSON array containing each line
# expressed as a JSON object - each line is split into columns separated by
# tab characters, the column names being passed as a comma separated list in
# $1.  Values are presented as strings, unless the associated column name is
# suffixed with ":I", in which case it will be an integer.
#
uactJsonObjectList () {
	rowFormat="$1"
	rawData="$2"

	if ! test -n "${rawData}"; then
		printf "%s" "[]"
		return
	fi

	printf "%s" "["
	{
	# Can't do sed 's,^,!!\t,;s,:,\t,' - OpenBSD sed doesn't do \t
	printf "%s\n" "${rowFormat}" | tr ',' '\n' | { while read -r nameLine; do printf "!!\t%s\n" "${nameLine}" | tr ':' '\t'; done; }
	printf "%s\n" "${rawData}"
	} \
	| awk -F "\t" '
BEGIN { firstLine=1; colCount=0 }
$1=="!!" { colCount=FNR; name[FNR]=$2; isInteger[FNR]=0; if ($3=="I") isInt[FNR]=1; }
$1!="!!" && $1!="" {
	if (firstLine) {
		printf "\n"
		firstLine=0
	} else {
		printf ",\n"
	}
	printf "%s", "    {"
	for (col=1; col<=colCount; col++) {
		if (col == 1) {
			printf "\n"
		} else {
			printf ",\n"
		}
		value=$col
		if (isInt[col]) {
			printf "      \"%s\": %d", name[col], value;
		} else {
			gsub(/\\/,"\\\\",value);
			gsub(/"/,"\\\"",value);
			gsub(/\r/,"\\r",value);
			printf "      \"%s\": \"%s\"", name[col], value;
		}
	}
	printf "\n%s", "    }"
}'
	printf "\n%s" "  ]"
}


# Output information about the completed action, including the action output
# ($1), as JSON.
#
uactJsonOutput () {
	rawOutput="$1"

	printf "{\n"
	printf "  \"action\": \"%s\",\n" "$(uactJsonEscapeString "${action}")"
	printf "  \"arguments\": \"%s\",\n" "$(uactJsonEscapeString "${actionArguments}")"
	printf "  \"exitStatus\": %d,\n" "${exitStatus}"
	test -n "${uactLastError}" && printf "  \"error\": \"%s\",\n" "$(uactJsonEscapeString "${uactLastError}")"
	case "${outputDataType}" in
	"strings") printf "  \"output\": %s\n" "$(uactJsonStringList "${rawOutput}")" ;;
	"string")  printf "  \"output\": \"%s\"\n" "$(uactJsonEscapeString "${rawOutput}")" ;;
	"boolean")
		case "${rawOutput}" in
		"true")  printf "  \"output\": %s\n" "true" ;;
		"false") printf "  \"output\": %s\n" "false" ;;
		*)       printf "  \"output\": %s\n" "null" ;;
		esac
		;;
	"list"*) printf "  \"output\": %s\n" "$(uactJsonObjectList "${outputDataType#list }" "${rawOutput}")" ;;
	esac
	printf "}\n"
}
