#!/bin/sh
#
# Functions to enumerate listening ports.
#
# Copyright 2024 Andrew Wood
#
# License GPLv3+: GNU GPL version 3 or later; see `docs/COPYING'.
#

# Report all listening TCP and UDP ports, writing record lines containing
# protocol (tcp|udp|tcp4|udp4|tcp6|udp6), bound IP (0.0.0.0 for any), port
# number, associated PID, and associated command.  If multiple processes are
# on the same port, one record will be produced for each process.
#
# The PID may be 0 if unknown, and the command may be "-".
#
uactListeningPorts () {
	systemType="$(uactRun uname -s | tr '[:upper:]' '[:lower:]')"

	rawData=""
	cmdExitStatus=0

	if command -v ss >/dev/null 2>&1 && ss --no-header --numeric >/dev/null 2>&1; then

		# ss ("socket statistics"), a version that's new enough to
		# have --no-header, since older ones (such as from
		# iproute2-ss061002 on CentOS 5) don't list any UDP
		# listeners.

		rawData="$(LC_ALL=C uactRun ss --listening --tcp --udp --numeric --processes 2>/dev/null)"
		cmdExitStatus=$?
		uactDebug "command exit status: ${cmdExitStatus}"

		# Columns: Netid State Recv-Q Send-Q Local_Address:Port Peer_Address:Port Process
		# Or: Netid Recv-Q Send-Q Local_Address:Port Peer_Address:Port Process
		# Process looks like: users:(("rpcbind",pid=457,fd=6),("systemd",pid=1,fd=49))
		# or it can be like: users:(("rpcbind",457,6),("systemd",1,49))

		printf "%s\n" "${rawData}" \
		| awk '
FNR==1 {
	hasStateColumn=0
	if (tolower($2)=="state")
		hasStateColumn=1
}
/^(tcp|udp)/ { 
	proto=$1
	boundTo=$4
	processesRaw=$6
	if (hasStateColumn) {
		boundTo=$5
		processesRaw=$7
	}

	port=boundTo
	sub(/^.*:/,"",port)

	ip=boundTo
	sub(/:[0-9]+$/,"",ip)
	sub(/%.*$/,"",ip)
	if (ip == "*")
		ip="0.0.0.0"

	sub(/^users:\(/,"",processesRaw)
	sub(/\)$/,"",processesRaw)
	processCount=split(processesRaw,processList,/\),/)
	for (arrayIdx=1; arrayIdx<=processCount; arrayIdx++) {
		processSpec=processList[arrayIdx]

		cmd=processSpec
		sub(/^\("/,"",cmd)
		sub(/",pid=.*$/,"",cmd)
		sub(/",[0-9].*$/,"",cmd)

		pid=processSpec
		sub(/^.*",pid=/,"",pid)
		sub(/^.*",/,"",pid)
		sub(/[^0-9].*$/,"",pid)

		printf "%s\t%s\t%s\t%s\t%s\n", proto, ip, port, pid, cmd
	}

	if (processCount < 1)
		printf "%s\t%s\t%s\t%s\t%s\n", proto, ip, port, "0", "-"
}
'
	
	elif test "${systemType}" = "freebsd" && command -v sockstat >/dev/null 2>&1; then

		# FreeBSD sockstat - because FreeBSD netstat doesn't do what
		# we want.

		rawData="$(LC_ALL=C uactRun sockstat -qnLl46 2>/dev/null)"
		cmdExitStatus=$?
		uactDebug "command exit status: ${cmdExitStatus}"

		# Columns: USER COMMAND PID FD PROTO LOCAL_ADDRESS FOREIGN_ADDRESS

		printf "%s\n" "${rawData}" \
		| awk '
$5~/^(tcp|udp)/ { 
	cmd=$2
	pid=$3
	proto=$5
	boundTo=$6

	port=boundTo
	sub(/^.*:/,"",port)

	ip=boundTo
	sub(/:[0-9]+$/,"",ip)
	sub(/%.*$/,"",ip)
	if (ip == "*")
		ip="0.0.0.0"

	printf "%s\t%s\t%s\t%s\t%s\n", proto, ip, port, pid, cmd
}
'

	elif test "${systemType}" = "openbsd" && command -v netstat >/dev/null 2>&1; then

		# OpenBSD netstat - doesn't show process information.

		rawData="$(LC_ALL=C uactRun netstat -nl 2>/dev/null)"
		cmdExitStatus=$?
		uactDebug "command exit status: ${cmdExitStatus}"

		# Columns: Proto Recv-Q Send-Q Local_Address Foreign_Address
		# but Local_Address separates IP and port with ".", not ":". 
		# Also the IPv6 address can be suffixed with %interface
		# before ending with .port.

		printf "%s\n" "${rawData}" \
		| awk '
$1~/^(tcp|udp)/ { 
	proto=$1
	boundTo=$4

	segmentCount=split(boundTo,segments,/\./)
	port=segments[segmentCount]

	ip=boundTo
	sub(/\.[0-9]+$/,"",ip)
	sub(/%.*$/,"",ip)
	if (ip == "*")
		ip="0.0.0.0"

	printf "%s\t%s\t%s\t%s\t%s\n", proto, ip, port, "0", "-"
}
'

	elif command -v netstat >/dev/null 2>&1; then

		# Linux netstat - only shows one listening PID per port.

		rawData="$(LC_ALL=C uactRun netstat -nlp 2>/dev/null)"
		cmdExitStatus=$?
		uactDebug "command exit status: ${cmdExitStatus}"

		# Columns: Proto Recv-Q Send-Q Local_Address Foreign_Address State PID/Program_name
		# except State can be empty for UDP

		printf "%s\n" "${rawData}" \
		| awk '
$1~/^(tcp|udp)/ { 
	proto=$1
	boundTo=$4
	processRaw=$6
	if (processRaw=="LISTEN")
		processRaw=$7

	port=boundTo
	sub(/^.*:/,"",port)

	ip=boundTo
	sub(/:[0-9]+$/,"",ip)
	sub(/%.*$/,"",ip)
	if (ip == "*")
		ip="0.0.0.0"

	pid=processRaw
	sub(/\/.*$/,"",pid)

	cmd=processRaw
	sub(/^[0-9]+\//,"",cmd)

	printf "%s\t%s\t%s\t%s\t%s\n", proto, ip, port, pid, cmd
}
'

	else
		uactError "UNSUPPORTED-ACTION" "${action}"
		cmdExitStatus=2
	fi

	return "${cmdExitStatus}"
}
