#!/bin/sh
#
# OpenBSD-specific package management functions.
#
# Copyright 2024 Andrew Wood
#
# License GPLv3+: GNU GPL version 3 or later; see `docs/COPYING'.
#

# Setup required for all OpenBSD package actions - set "packageManagerFlags"
# according to whether interactive mode should be used; and set
# "packageListArgs" to the sanitised arguments.
#
uactOpenbsdPackage () {
	packageManagerFlags=""
	packageListArgs=""

	# Set the arguments according to whether interactive mode is
	# enabled.
	test "${optionNonInteractive}" -eq 1 && packageManagerFlags="-I"

	# Sanitise the input arguments.
	packageListArgs="$(uactSanitisePackageList "$*")"
}

# Install packages $* (actually ${packageListArgs}).
uactOpenbsdPackageInstall () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Upgrade packages $* (actually ${packageListArgs}).
uactOpenbsdPackageUpgrade () {
	uactRun "${packageManagerCommand}" -u ${packageManagerFlags} ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Downgrade packages $* (actually ${packageListArgs}).
uactOpenbsdPackageDowngrade () {
	uactRun "${packageManagerCommand}" -D downgrade ${packageManagerFlags} ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Remove packages $* (actually ${packageListArgs}).
uactOpenbsdPackageRemove () {
	uactRun pkg_delete ${packageManagerFlags} ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Reinstall packages $* (actually ${packageListArgs}).
uactOpenbsdPackageReinstall () {
	uactRun "${packageManagerCommand}" -u -D installed ${packageManagerFlags} ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Output information about packages $* (actually ${packageListArgs}), one
# per line, tab separated columns - package, architecture, version, install
# epoch time, vendor, type, summary.
#
uactOpenbsdPackageInfo () {
	uactRun pkg_info -q -c -f ${packageListArgs} \
	| awk -v OFS="\r" '
/^$/ && name~/[A-Za-z]/ { print name,arch,version,installed,vendor,"tgz",summary; name=""; summary="?" }
/^@name/ {
  name=$2
  sub(/-[^-]+$/,"",name)
  version=$2
  sub(/^.*-/,"",version)
  arch="?"
  pkgInfoDir=$2
  gsub(/[^0-9A-Za-z,_.-]/,"",pkgInfoDir)
  cmd="stat -f %m /var/db/pkg/" pkgInfoDir " 2>/dev/null"
  cmd | getline installed
  close(cmd)
  if (installed=="")
    installed=0
  vendor="?"
}
/^@arch/ { arch=$2 }
/^@signer/ { vendor=$2 }
/^[0-9A-Za-z]/ && name!~/[A-Za-z]/ { summary=$0 }
' \
	| tr '\t' ' ' \
	| tr '\r' '\t'
	return 0
}

# List the files owned by package $1 (actually ${packageListArgs}).
uactOpenbsdPackageFiles () {
	queryOutput="$(uactRun pkg_info -q -L ${packageListArgs})"
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	printf "%s\n" "${queryOutput}" | grep -E .
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List the package which owns file $1.
uactOpenbsdPackageOwningFile () {
	queryOutput=$(uactRun pkg_info -q -E "$1")
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 || return 1
	printf "%s\n" "${queryOutput}" | sed 's,-[^-]*$,,' | sort -u
	return 0
}

# Refresh the cache.
uactOpenbsdPackageCacheUpdate () {
	# This is unsupported on OpenBSD.
	uactError "UNSUPPORTED-ACTION" "${action}"
	return 2
}

# Clean all caches.
uactOpenbsdPackageCacheClean () {
	# This is unsupported on OpenBSD.
	uactError "UNSUPPORTED-ACTION" "${action}"
	return 2
}

# List packages ready to upgrade, one per line, tab separated columns -
# package, architecture, target version, repository, type.
uactOpenbsdPackageUpgradeList () {
	upgradeOutput="$(LC_ALL=C uactRun pkg_add -s -m -u | tr '\r' '\n' | awk '$1~/->/ && $NF=="ok" {print $1}')"
	upgradePackageList="$(printf "%s\n" "${upgradeOutput}" | sed 's,->.*$,,;s,-[^-]*$,,')"
	test -n "${upgradePackageList}" || return 0

	packageListArgs="${upgradePackageList}"
	packageQueryResult="$(uactOpenbsdPackageInfo ${upgradePackageList} 2>/dev/null | awk -F "\t" '{print $1,$2}')"

	printf "%s\n" "${upgradeOutput}" \
	| {
		while read -r packageName; do
			targetVersion="${packageName##*->}"
			targetVersion="${targetVersion%:}"
			packageName="${packageName%->*}"
			packageName="${packageName%-*}"
			packageArchitecture="$(printf "%s\n" "${packageQueryResult}" | awk -v "name=${packageName}" '$1==name {print $2}')"
			packageRepository="-"
			test -n "${packageArchitecture}" || packageArchitecture="?"
			printf "%s\t%s\t%s\t%s\t%s\n" "${packageName}" "${packageArchitecture}" "${packageRepository}" "${targetVersion}" "tgz"
		done
	}

	return 0
}

# Upgrade all packages.
uactOpenbsdPackageUpgradeAll () {
	uactRun "${packageManagerCommand}" -u ${packageManagerFlags}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List all installed packages.
uactOpenbsdPackageListInstalled () {
	uactOpenbsdPackageInfo
	return $?
}
