#!/bin/sh
#
# Service management functions specific to OpenRC.
#
# Copyright 2024 Andrew Wood
#
# License GPLv3+: GNU GPL version 3 or later; see `docs/COPYING'.
#

# Start service $1.
uactOpenrcServiceStart () {
	uactRun rc-service --quiet "$1" start
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Stop service $1.
uactOpenrcServiceStop () {
	uactRun rc-service --quiet "$1" stop
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Stop, then start, service $1.
uactOpenrcServiceRestart () {
	uactRun rc-service --quiet "$1" restart
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Restart service $1 if it is currently running.
uactOpenrcServiceTryRestart () {
	uactRun rc-service --quiet --ifstarted "$1" restart
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Tell service $1 to reload its configuration.
uactOpenrcServiceReload () {
	uactRun rc-service --quiet "$1" reload
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Enable service $1 so it will start in the current runlevel.
uactOpenrcServiceEnable () {
	currentRunlevel="$(uactRun rc-status --runlevel)"
	uactRun rc-update --quiet add "$1" "${currentRunlevel}"
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Disable service $1 so it will not start in the current runlevel.
uactOpenrcServiceDisable () {
	currentRunlevel="$(uactRun rc-status --runlevel)"
	uactRun rc-update --quiet delete "$1" "${currentRunlevel}"
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Mask service $1 so it will not start automatically and cannot be started.
uactOpenrcServiceMask () {
	# There is no mechanism to mask services with OpenRC.
	uactError "UNSUPPORTED-ACTION" "${action}"
	return 2
}

# Unmask service $1 so it can be started and may start automatically.
uactOpenrcServiceUnmask () {
	# There is no mechanism to mask services with OpenRC.
	uactError "UNSUPPORTED-ACTION" "${action}"
	return 2
}

# Report "true" or "false" depending on whether service $1 exists.
uactOpenrcServiceIsKnown () {
	uactRun rc-service --list 2>/dev/null | grep -Fqx "$1" && { echo "true"; return 0; }
	echo "false"
	return 1
}

# Report "true" or "false" depending on whether service $1 is running.
uactOpenrcServiceIsRunning () {
	LC_ALL=C uactRun rc-status --format ini 2>/dev/null \
	| awk '/=/ && $NF=="started" { print $1 }' \
	| grep -Fqx "$1" \
	&& { echo "true"; return 0; }
	echo "false"
	return 1
}

# Report "true" or "false" depending on whether service $1 is enabled in the
# current runlevel.
uactOpenrcServiceIsEnabled () {
	currentRunlevel="$(uactRun rc-status --runlevel)"
	uactRun rc-update --quiet show "${currentRunlevel}" 2>/dev/null \
	| awk '{print $1}' \
	| grep -Fqx "$1" \
	&& { echo "true"; return 0; }
	echo "false"
	return 1
}

# Report "true" or "false" depending on whether service $1 is masked.
uactOpenrcServiceIsMasked () {
	# There is no mechanism to mask services with OpenRC.
	uactError "UNSUPPORTED-ACTION" "${action}"
	return 2
}

# Output one line per service containing the fields "service" (name), "boot"
# (enabled, disabled, masked), "active" (inactive, activating, active),
# "description", TAB separated.
uactOpenrcServiceListAll () {
	currentRunlevel="$(uactRun rc-status --runlevel)"
	enabledServices="$(uactRun rc-update --quiet show "${currentRunlevel}" 2>/dev/null | awk '{print $1}')"
	startedServices="$(LC_ALL=C uactRun rc-status --format ini 2>/dev/null | awk '/=/ && $NF=="started" { print $1 }')"
	uactRun rc-service --list | sort -u | \
	{
	while read -r serviceName; do
		test -n "${serviceName}" || continue

		bootStatus="disabled"
		printf "%s\n" "${enabledServices}" | grep -Fqx "${serviceName}" && bootStatus="enabled"

		activeStatus="inactive"
		printf "%s\n" "${startedServices}" | grep -Fqx "${serviceName}" && activeStatus="active"

		serviceDescription="$(LC_ALL=C uactRun rc-service "${serviceName}" describe 2>&1 \
		  | sed -n '1{s/^ \* //;p}' \
		  | grep -Ev '^No description for ' \
		  | tr '\t\n' '  ')"
		test -n "${serviceDescription}" || serviceDescription="-"

		printf "%s\t%s\t%s\t%s\n" "${serviceName}" "${bootStatus}" "${activeStatus}" "${serviceDescription}"
	done
	}
}
