#!/bin/sh
#
# Functions for loading override functions and finding their names.
#
# Copyright 2023-2024 Andrew Wood
#
# License GPLv3+: GNU GPL version 3 or later; see `docs/COPYING'.
#

# Given an override prefix $1 - such as "debian", for the package manager
# type; a script base name $2, such as "package" or "package-install" (from
# $actionClass or $action); and function name $3, such as
# "uactPackageInstall": load the script $1-$2.sh if it exists, and if the
# override function of the form "uactDebianPackageInstall" exists, set
# "overrideFunction" to it, otherwise set it blank.
#
uactFindOverride () {
	overridePrefix="$1"
	originalScript="$2"
	originalFunction="$3"

	if test -e "${COMPONENTDIR}/${overridePrefix}-${originalScript}.sh"; then
		uactDebug "loading override script: ${COMPONENTDIR}/${overridePrefix}-${originalScript}.sh"
		. "${COMPONENTDIR}/${overridePrefix}-${originalScript}.sh"
	fi

	overrideFunction=""
	prefixUcFirst="$(echo "${overridePrefix}" | cut -b 1 | tr '[:lower:]' '[:upper:]')${overridePrefix#?}"
	overrideFunction="uact${prefixUcFirst}${originalFunction#uact}"
	type "${overrideFunction}" >/dev/null 2>&1 || overrideFunction=""

	return 0
}

# Given a type prefix, such as "debian", run its override class function for
# the current $actionClassFunction if there is one (e.g. 
# "uactDebianPackage"); then look for an override function for the current
# $actionFunction, and if one exists, set actionFunction to it (e.g. 
# "uactDebianPackageRemove").
#
uactOverrideClass () {
	typePrefix="$1"
	shift

	# Load the code relevant to this type of package manager, and find
	# the override functions.
	uactFindOverride "${typePrefix}" "${actionClass}" "${actionClassFunction}"
	overrideClassFunction="${overrideFunction}"
	uactFindOverride "${typePrefix}" "${action}" "${actionFunction}"
	overrideActionFunction="${overrideFunction}"

	# Call the override class function if it was found.
	if test -n "${overrideClassFunction}"; then
		uactDebug "running action class override function: ${overrideClassFunction}"
		"${overrideClassFunction}" "$@"
		subReturnCode=$?
		test "${subReturnCode}" -gt 0 && return "${subReturnCode}"
	fi

	# Override the action function if an override for it was found, so
	# that the caller will call that, instead of the original.
	if test -n "${overrideActionFunction}"; then
		uactDebug "replacing action function with action override function: ${overrideActionFunction}"
		actionFunction="${overrideActionFunction}"
	fi

	return 0
}
