#!/bin/sh
#
# Code common to all package actions.
#
# Copyright 2023-2024 Andrew Wood
#
# License GPLv3+: GNU GPL version 3 or later; see `docs/COPYING'.
#

packageManagerType=""
packageManagerCommand=""
packageInfoTool=""

# Set the "packageManagerType" variable to the function prefix for the
# package manager that is in use on this system, "packageManagerCommand" to
# the name of the main management command, and "packageInfoTool" to the name
# of the individual package tool command, or report an error if no supported
# package manager was found (and return 2).
#
uactIdentifyPackageManager () {
	if command -v apt-get >/dev/null 2>&1; then
		packageManagerType="debian"
		packageManagerCommand="apt-get"
		packageInfoTool="dpkg-query"
	elif command -v zypper >/dev/null 2>&1; then
		packageManagerType="suse"
		packageManagerCommand="zypper"
		packageInfoTool="rpm"
	elif command -v dnf >/dev/null 2>&1; then
		packageManagerType="redhat"
		packageManagerCommand="dnf"
		packageInfoTool="rpm"
	elif command -v yum >/dev/null 2>&1; then
		packageManagerType="redhat"
		packageManagerCommand="yum"
		packageInfoTool="rpm"
	elif command -v pkg_add >/dev/null 2>&1; then
		packageManagerType="openbsd"
		packageManagerCommand="pkg_add"
	elif command -v pkg >/dev/null 2>&1; then
		packageManagerType="freebsd"
		packageManagerCommand="pkg"
	elif command -v pacman >/dev/null 2>&1; then
		packageManagerType="archlinux"
		packageManagerCommand="pacman"
	elif command -v apk >/dev/null 2>&1; then
		packageManagerType="alpine"
		packageManagerCommand="apk"
	else
		uactError "NO-PACKAGE-MANAGER"
		return 2
	fi
	uactDebug "packageManagerType=[${packageManagerType}], packageManagerCommand=[${packageManagerCommand}], packageInfoTool=[${packageInfoTool}]"
	return 0
}

# Sanitise the given list of package names to strip those containing / or ;
# characters, and any starting with "-".
#
uactSanitisePackageList () {
	echo "$*" | tr ' \t' '\n\n' | grep -Ev -e '^-' -e '/' -e ';' | tr -s '\n' | tr '\n' ' ' | sed 's/ $//'
}

# Catch-all for all package actions.  Load the code appropriate to the
# package manager, run its action class function, and override the action
# function.
#
uactPackage () {
	uactIdentifyPackageManager
	subReturnCode=$?
	test "${subReturnCode}" -eq 0 || return "${subReturnCode}"

	# Load the appropriate overrides for the package type.
	uactOverrideClass "${packageManagerType}" "$@"
	subReturnCode=$?
	test "${subReturnCode}" -eq 0 || return "${subReturnCode}"

	return 0
}
