#!/bin/sh
#
# Package management functions for Red Hat and derivatives.
#
# Copyright 2024 Andrew Wood
#
# License GPLv3+: GNU GPL version 3 or later; see `docs/COPYING'.
#

# Setup required for all Red Hat package actions - set "packageManagerFlags"
# according to whether interactive mode should be used; and set
# "packageListArgs" to the sanitised arguments.
#
uactRedhatPackage () {
	packageManagerFlags=""
	packageListArgs=""

	# Set the arguments according to whether interactive mode is
	# enabled.
	test "${optionNonInteractive}" -eq 1 && packageManagerFlags="-y"

	# Sanitise the input arguments.
	packageListArgs="$(uactSanitisePackageList "$*")"
}

# Install packages $* (actually ${packageListArgs}).
uactRedhatPackageInstall () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} install ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Upgrade packages $* (actually ${packageListArgs}).
uactRedhatPackageUpgrade () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} update ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Downgrade packages $* (actually ${packageListArgs}).
uactRedhatPackageDowngrade () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} downgrade ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Remove packages $* (actually ${packageListArgs}).
uactRedhatPackageRemove () {
	uactRun "${packageInfoTool}" -e ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Reinstall packages $* (actually ${packageListArgs}).
uactRedhatPackageReinstall () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} reinstall ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Output information about packages $* (actually ${packageListArgs}), one
# per line, tab separated columns - package, architecture, version, install
# epoch time, vendor, type, summary.
#
uactRedhatPackageInfo () {
	# Use \r as a field separator since it's unlikely to appear in the
	# name or synopsis, so we can strip tabs from inside columns.
	uactRun "${packageInfoTool}" -qa --queryformat='%{NAME}\r%{ARCH}\r%{VERSION}-%{RELEASE}\r%{INSTALLTIME}\r%{VENDOR}\rrpm\r%{SUMMARY}\n' ${packageListArgs} 2>/dev/null \
	| tr '\t' ' ' \
	| tr '\r' '\t'
	return 0
}

# List the files owned by package $* (actually ${packageListArgs}).
uactRedhatPackageFiles () {
	uactRun "${packageInfoTool}" -ql ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List the package which owns file $1.
uactRedhatPackageOwningFile () {
	queryOutput=$(uactRun "${packageInfoTool}" -q --queryformat='%{NAME}' -f "$1")
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 || return 1
	printf "%s\n" "${queryOutput}" | sort -u
	return 0
}

# Refresh the cache.
uactRedhatPackageCacheUpdate () {
	uactRun "${packageManagerCommand}" makecache
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Clean all caches.
uactRedhatPackageCacheClean () {
	uactRun "${packageManagerCommand}" clean all
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List packages ready to upgrade.
uactRedhatPackageUpgradeList () {
	queryOutput=$(LC_ALL=C uactRun "${packageManagerCommand}" check-update 2>/dev/null)
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 1 && return 1
	test "${cmdExitStatus}" -eq 0 && return 0
	# package, arch, target-version, repo, type
	printf "%s\n" "${queryOutput}" \
	| grep -Ev '^Security: ' \
	| sed '1,/^$/d' \
	| awk '{match($1,/^(.+)\.([^.]+)$/,a);if (a[2] == "") { a[1]=$1; a[2]="-" } printf "%s\t%s\t%s\t%s\t%s\n",a[1],a[2],$2,$3,"rpm"}' \
	| sort -u
	return 0
}

# Upgrade all packages.
uactRedhatPackageUpgradeAll () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} update
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List all installed packages.
uactRedhatPackageListInstalled () {
	uactRedhatPackageInfo
	return $?
}
