#!/bin/sh
#
# SUSE-specific package management functions.
#
# Copyright 2024 Andrew Wood
#
# License GPLv3+: GNU GPL version 3 or later; see `docs/COPYING'.
#

# Setup required for all SUSE package actions - set "packageManagerFlags"
# according to whether interactive mode should be used; and set
# "packageListArgs" to the sanitised arguments.
#
uactSusePackage () {
	packageManagerFlags=""
	packageListArgs=""

	# Set the arguments according to whether interactive mode is
	# enabled.
	test "${optionNonInteractive}" -eq 1 && packageManagerFlags="--non-interactive"

	# Sanitise the input arguments.
	packageListArgs="$(uactSanitisePackageList "$*")"
}

# Install packages $* (actually ${packageListArgs}).
uactSusePackageInstall () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} install ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Upgrade packages $* (actually ${packageListArgs}).
uactSusePackageUpgrade () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} update ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Downgrade packages $* (actually ${packageListArgs}).
uactSusePackageDowngrade () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} update --allow-downgrade ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Remove packages $* (actually ${packageListArgs}).
uactSusePackageRemove () {
	uactRun "${packageInfoTool}" -e ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Reinstall packages $* (actually ${packageListArgs}).
uactSusePackageReinstall () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} install --force ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Output information about packages $* (actually ${packageListArgs}), one
# per line, tab separated columns - package, architecture, version, install
# epoch time, vendor, type, summary.
#
uactSusePackageInfo () {
	# Use \r as a field separator since it's unlikely to appear in the
	# name or synopsis, so we can strip tabs from inside columns.
	uactRun "${packageInfoTool}" -qa --queryformat='%{NAME}\r%{ARCH}\r%{VERSION}-%{RELEASE}\r%{INSTALLTIME}\r%{VENDOR}\rrpm\r%{SUMMARY}\n' ${packageListArgs} 2>/dev/null \
	| tr '\t' ' ' \
	| tr '\r' '\t'
	return 0
}

# List the files owned by package $* (actually ${packageListArgs}).
uactSusePackageFiles () {
	uactRun "${packageInfoTool}" -ql ${packageListArgs}
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List the package which owns file $1.
uactSusePackageOwningFile () {
	queryOutput=$(uactRun "${packageInfoTool}" -q --queryformat='%{NAME}' -f "$1")
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 || return 1
	printf "%s\n" "${queryOutput}" | sort -u
	return 0
}

# Refresh the cache.
uactSusePackageCacheUpdate () {
	uactRun "${packageManagerCommand}" refresh
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Clean all caches.
uactSusePackageCacheClean () {
	uactRun "${packageManagerCommand}" clean --all
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List packages ready to upgrade.
uactSusePackageUpgradeList () {
	queryOutput=$(LC_ALL=C uactRun "${packageManagerCommand}" --non-interactive --terse --table-style 10 list-updates 2>/dev/null)
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 || return 1
	printf "%s\n" "${queryOutput}" \
	| awk '{match($0,/^. *: *([^: ][^:]+[^: ]+) *: *([^: ]+) *: *([^: ]+) *: *([^: ]+) *: *([^: ]+) *$/,a);if (a[2] != "") printf "%s\t%s\t%s\t%s\t%s\n",a[2],a[5],a[4],a[1],"rpm"}' \
	| sort -u
	return 0
}

# Upgrade all packages.
uactSusePackageUpgradeAll () {
	uactRun "${packageManagerCommand}" ${packageManagerFlags} update
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# List all installed packages.
uactSusePackageListInstalled () {
	uactSusePackageInfo
	return $?
}
