#!/bin/sh
#
# Service management functions specific to systemd.
#
# Copyright 2023-2024 Andrew Wood
#
# License GPLv3+: GNU GPL version 3 or later; see `docs/COPYING'.
#

# Start service $1.
uactSystemdServiceStart () {
	uactRun systemctl --quiet start "$1"
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Stop service $1.
uactSystemdServiceStop () {
	uactRun systemctl --quiet stop "$1"
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Stop, then start, service $1.
uactSystemdServiceRestart () {
	uactRun systemctl --quiet restart "$1"
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Restart service $1 if it is currently running.
uactSystemdServiceTryRestart () {
	uactRun systemctl --quiet try-restart "$1"
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Tell service $1 to reload its configuration.
uactSystemdServiceReload () {
	uactRun systemctl --quiet reload "$1"
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Enable service $1 so it will start on boot.
uactSystemdServiceEnable () {
	uactRun systemctl --quiet enable "$1"
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Disable service $1 so it will not start on boot.
uactSystemdServiceDisable () {
	uactRun systemctl --quiet disable "$1"
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Mask service $1 so it will not start on boot and cannot be started.
uactSystemdServiceMask () {
	uactRun systemctl --quiet mask "$1"
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Unmask service $1 so it can be started and may start on boot.
uactSystemdServiceUnmask () {
	uactRun systemctl --quiet unmask "$1"
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Report "true" or "false" depending on whether service $1 exists.
uactSystemdServiceIsKnown () {
	uactRun systemctl is-enabled "$1" 2>/dev/null | grep -Eq . && { echo "true"; return 0; }
	echo "false"
	return 1
}

# Report "true" or "false" depending on whether service $1 is running.
uactSystemdServiceIsRunning () {
	uactRun systemctl --quiet is-active "$1" 2>/dev/null && { echo "true"; return 0; }
	echo "false"
	return 1
}

# Report "true" or "false" depending on whether service $1 is enabled.
uactSystemdServiceIsEnabled () {
	uactRun systemctl --quiet is-enabled "$1" 2>/dev/null && { echo "true"; return 0; }
	echo "false"
	return 1
}

# Report "true" or "false" depending on whether service $1 is masked.
uactSystemdServiceIsMasked () {
	LC_ALL=C uactRun systemctl is-enabled "$1" 2>/dev/null | grep -Fq masked && { echo "true"; return 0; }
	echo "false"
	return 1
}

# Output one line per service containing the fields "service" (name), "boot"
# (enabled, disabled, masked), "active" (inactive, activating, active),
# "description", TAB separated.
uactSystemdServiceListAll () {

	# When listing services with "systemctl", there can be discrepancies
	# in the listings between the running and enabled status in terms of
	# services being missed out, as well as the complication of legacy
	# SysVinit services, so we have to do a couple of lookups and stick
	# them together.

	workDir=$(mktemp -d) || return 2
	trap 'rm -rf "${workDir}"' EXIT

	# Whether each service is running (including legacy SysVinit).
	LC_ALL=C uactRun systemctl list-units --type=service --no-pager --no-legend --all | sed 's/^..//;s/\.service//' > "${workDir}/runningStatus"

	# Whether each service is enabled (this doesn't include legacy sysVinit services).
	LC_ALL=C uactRun systemctl list-unit-files --type=service --no-pager --no-legend --all | awk '$2!="alias" {print $1 " " $2}' | sed 's/\.service//' | sed 's/ *$//;s/-runtime$//' > "${workDir}/enableStatus"

	# Whether each legacy sysVinit service is enabled.
	sysVrunlevel=""
	command -v runlevel >/dev/null 2>&1 && sysVrunlevel=$(uactRun runlevel 2>/dev/null | awk '{print $2}')
	if test -n "${sysVrunlevel}" && command -v chkconfig >/dev/null 2>&1; then
		LC_ALL=C uactRun chkconfig --list 2>&1 | sed -n 's/ .*'"${sysVrunlevel}"':\(off\|on\).*$/ \1/p' | sed 's, off$, disabled,;s, on$, enabled,' >> "${workDir}/enableStatus"
	elif test -n "${sysVrunlevel}"; then
		LC_ALL=C uactRun ls /etc/init.d/ 2>/dev/null | sort > "${workDir}/allSysVinitScripts"
		LC_ALL=C uactRun ls /etc/rc"${sysVrunlevel}".d/ 2>/dev/null | sed -n 's/^S..//p' | sort > "${workDir}/enabledSysVinitScripts"
		comm -23 "${workDir}/allSysVinitScripts" "${workDir}/enabledSysVinitScripts" | awk '{print $1 " disabled"}' >> "${workDir}/enableStatus"
		comm -13 "${workDir}/allSysVinitScripts" "${workDir}/enabledSysVinitScripts" | awk '{print $1 " enabled"}' >> "${workDir}/enableStatus"
	fi

	# Services to include information about.
	grep -E '(enabled|disabled|masked)$' "${workDir}/enableStatus" 2>/dev/null | awk '{print $1}' | tr -d '@' | sort -u > "${workDir}/services"

	# Separate out the running status and descriptions.
	exec 4<&0
	exec <"${workDir}/runningStatus"
	while read -r serviceName isLoaded runStatus subStatus description; do
		test "${isLoaded}" = "loaded" || continue
		printf "%s\t%s\n" "${serviceName%.service}" "${runStatus}" >> "${workDir}/allRunStatus"
		printf "%s\t%s\n" "${serviceName%.service}" "${description}" >> "${workDir}/allDescription"
	done
	exec 0<&4

	# Some services don't appear in list-units, such as httpd, for no
	# apparent reason, so look them up separately.
	#
	awk '{print $1}' "${workDir}/runningStatus" | sort -u > "${workDir}/runningStatusServices"
	comm -13 "${workDir}/runningStatusServices" "${workDir}/services" | awk '{print $1}' > "${workDir}/missingServices"
	exec 4<&0
	exec <"${workDir}/missingServices"
	while read -r serviceName; do
		uactRun systemctl show --property=ActiveState,Description "${serviceName}" \
		| awk -F '=' '$1=="Description"{desc=$2} $1=="ActiveState"{actst=$2} END { print actst " " desc }' \
		| while read -r runStatus description; do
			printf "%s\t%s\n" "${serviceName%.service}" "${runStatus}" >> "${workDir}/allRunStatus"
			printf "%s\t%s\n" "${serviceName%.service}" "${description}" >> "${workDir}/allDescription"
		done
	done
	exec 0<&4

	# Produce the formatted output.
	exec 4<&0
	exec < "${workDir}/services"
	while read -r serviceName; do
		enableStatus=$(awk -v srv="${serviceName}" '$1==srv { print $2 }' "${workDir}/enableStatus" | sed -n 1p)
		test "${enableStatus}" = "static" && enableStatus="disabled"
		runStatus=$(awk -F "\t" -v srv="${serviceName}" '$1==srv { print $2 }' "${workDir}/allRunStatus" | sed -n 1p)
		description=$(awk -F "\t" -v srv="${serviceName}" '$1==srv { print $2 }' "${workDir}/allDescription" | sed -n 1p)
		printf "%s\t%s\t%s\t%s\n" "${serviceName}" "${enableStatus}" "${runStatus}" "${description}"
	done
	exec 0<&4

	rm -rf "${workDir}"
	trap '' EXIT

	return 0
}
