#!/bin/sh
#
# Service management functions specific to System V init.
#
# Copyright 2024 Andrew Wood
#
# License GPLv3+: GNU GPL version 3 or later; see `docs/COPYING'.
#

# Start service $1.
uactSysvinitServiceStart () {
	uactRun service "$1" start
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Stop service $1.
uactSysvinitServiceStop () {
	uactRun service "$1" stop
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Stop, then start, service $1.
uactSysvinitServiceRestart () {
	uactRun service "$1" restart
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Restart service $1 if it is currently running.
uactSysvinitServiceTryRestart () {
	uactRun service "$1" condrestart >/dev/null
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Tell service $1 to reload its configuration.
uactSysvinitServiceReload () {
	uactRun service "$1" reload >/dev/null
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Enable service $1 so it will start on boot.
uactSysvinitServiceEnable () {
	if test "${serviceManagerVariant}" = "debian"; then
		uactRun update-rc.d "$1" enable >/dev/null
		cmdExitStatus=$?
	elif test "${serviceManagerVariant}" = "redhat"; then
		uactRun chkconfig "$1" on >/dev/null
		cmdExitStatus=$?
	else
		uactError "NO-SERVICE-MANAGER"
		return 2
	fi
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Disable service $1 so it will not start on boot.
uactSysvinitServiceDisable () {
	if test "${serviceManagerVariant}" = "debian"; then
		uactRun update-rc.d "$1" disable >/dev/null
		cmdExitStatus=$?
	elif test "${serviceManagerVariant}" = "redhat"; then
		uactRun chkconfig "$1" off >/dev/null
		cmdExitStatus=$?
	else
		uactError "NO-SERVICE-MANAGER"
		return 2
	fi
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Mask service $1 so it will not start on boot and cannot be started.
uactSysvinitServiceMask () {
	uactRun chmod 000 "/etc/init.d/$1"
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Unmask service $1 so it can be started and may start on boot.
uactSysvinitServiceUnmask () {
	uactRun chmod 755 "/etc/init.d/$1"
	cmdExitStatus=$?
	uactDebug "command exit status: ${cmdExitStatus}"
	test "${cmdExitStatus}" -eq 0 && return 0
	return 1
}

# Report "true" or "false" depending on whether service $1 exists.
uactSysvinitServiceIsKnown () {
	uactRun test -e "/etc/init.d/$1" && { echo "true"; return 0; }
	echo "false"
	return 1
}

# Report "true" or "false" depending on whether service $1 is running.
uactSysvinitServiceIsRunning () {
	uactRun service "$1" status >/dev/null && { echo "true"; return 0; }
	echo "false"
	return 1
}

# Report "true" or "false" depending on whether service $1 is enabled.
uactSysvinitServiceIsEnabled () {
	sysVrunlevel=""
	command -v runlevel >/dev/null 2>&1 && sysVrunlevel="$(uactRun runlevel 2>/dev/null | awk '{print $2}')"
	test -n "${sysVrunlevel}" || sysVrunlevel="3"
	uactRun find "/etc/rc${sysVrunlevel}.d/" -type l -name "S??$1" 2>/dev/null | grep -Eq . && { echo "true"; return 0; }
	echo "false"
	return 1
}

# Report "true" or "false" depending on whether service $1 is masked.
uactSysvinitServiceIsMasked () {
	uactRun test -e "/etc/init.d/$1" || { echo "false"; return 1; }
	uactRun test -x "/etc/init.d/$1" && { echo "false"; return 1; }
	echo "true"
	return 0
}

# Output one line per service containing the fields "service" (name), "boot"
# (enabled, disabled, masked), "active" (inactive, activating, active),
# "description", TAB separated.
uactSysvinitServiceListAll () {

	workDir="$(mktemp -d)" || return 2
	trap 'rm -rf "${workDir}"' EXIT

	sysVrunlevel=""
	command -v runlevel >/dev/null 2>&1 && sysVrunlevel="$(uactRun runlevel 2>/dev/null | awk '{print $2}')"
	test -n "${sysVrunlevel}" || sysVrunlevel="3"

	# List all known services, and whether they are enabled on boot.
	uactRun find "/etc/rc${sysVrunlevel}.d/" -type l -name "[SK]??*" 2>/dev/null \
	| awk -F / '{file=$NF; service=file; sub(/^.../,"",service); print service, file~/^S/ ? "enabled" : "disabled" }' \
	| sort \
	> "${workDir}/enableStatus"

	# Produce the formatted output.
	{
	while read -r serviceName bootStatus; do
		test -e "/etc/init.d/${serviceName}" || continue
		test -x "/etc/init.d/${serviceName}" || bootStatus="masked"

		activeStatus="inactive"
		uactSysvinitServiceIsRunning "${serviceName}" >/dev/null && activeStatus="active"

		serviceDescription=""
		serviceDescription="$(grep -Ei '^(# *)? *short-description:' "/etc/init.d/${serviceName}" 2>/dev/null | cut -d : -f 2- | sed 's/^ *//' | sed -n 1p)"
		test -n "${serviceDescription}" || serviceDescription="$(grep -Ei '^(# *)? *description:' "/etc/init.d/${serviceName}" 2>/dev/null | cut -d : -f 2- | sed 's/^ *//' | sed -n 1p | sed 's/\\$/.../')"
		serviceDescription="$(printf "%s\n" "${serviceDescription}" | tr '\t' ' ')"
		test -n "${serviceDescription}" || serviceDescription="-"

		printf "%s\t%s\t%s\t%s\n" "${serviceName}" "${bootStatus}" "${activeStatus}" "${serviceDescription}"
	done
	} < "${workDir}/enableStatus"

	rm -rf "${workDir}"
	trap '' EXIT

	return 0
}
